#ifndef TETRIS_BLOCK_H
#define TETRIS_BLOCK_H
z
#include <memory>
#include <vector>
#include <SFML/Graphics.hpp>

enum class Shape : uint8_t {
    I=1,//4
    T, //2
    O, //2
    L, //3
    J, //3
    S, //2
    Z  //2
};

class InterfaceBlock {
public:
    virtual void create(const Shape& m_shape) = 0;
    virtual void rotate() = 0;
    virtual std::vector<sf::RectangleShape>& draw() = 0;
    virtual ~InterfaceBlock() {};

private:

};

class Block : virtual public InterfaceBlock {
public:
    Block(const Shape& shape);
    ~Block() {};
    void create(const Shape& m_shape) override;
    void rotate() override;
    void init() { create(m_shape); };
    std::vector<sf::RectangleShape>& draw() override;
    void setPosition(const std::vector<sf::Vector2f>& vec);
    [[nodiscard]] const std::vector<sf::Vector2f>& getPosition() const;
    [[nodiscard]] Shape getShape() const { return m_shape; };
    std::vector<uint8_t> getBlocks() const { return m_blocks; }
    void setBlocks(const std::vector<uint8_t>& blocks);
    friend bool operator==(const Block& lhs, const Block& rhs) {
        if (lhs.m_shape == rhs.m_shape && lhs.m_blocks==rhs.m_blocks
            && lhs.pos==rhs.pos) return true;
        return false;
    };
    
private:
    Shape m_shape;
    std::vector<uint8_t> m_blocks;
    std::vector<sf::RectangleShape> m_shapes;
    std::vector<sf::Vector2f> pos;
};

class BlockFactory {
    virtual std::unique_ptr<Block> create() = 0;
};

#endif //TETRIS_BLOCK_H
